#ifndef _QDPIO_H
#define _QDPIO_H

#include <qio.h>
#include <qioxml.h>
#include <qdp_string.h>

/* volume formats */
#define QDP_SINGLEFILE QIO_SINGLEFILE 
#define QDP_MULTIFILE  QIO_MULTIFILE  

#ifdef __cplusplus
extern "C" {
#endif

typedef struct QDP_Reader_struct QDP_Reader;
typedef struct QDP_Writer_struct QDP_Writer;

QDP_Writer *QDP_open_write(QDP_String *md, char *filename, int volfmt);
int QDP_close_write(QDP_Writer *qw);

QDP_Reader *QDP_open_read(QDP_String *md, char *filename);
int QDP_close_read(QDP_Reader *qr);

int QDP_read_record_info(QDP_Reader *qdpr, QDP_String *md);
int QDP_next_record(QDP_Reader *qr);

/* Read and write single field */

int QDP_read_S( QDP_Reader *qr, QDP_String *md, QDP_RandomState *field);
int QDP_write_S( QDP_Writer *qw, QDP_String *md, QDP_RandomState *field);
int QDP_read_I( QDP_Reader *qr, QDP_String *md, QDP_Int *field);
int QDP_write_I( QDP_Writer *qw, QDP_String *md, QDP_Int *field);
int QDP_F_read_R( QDP_Reader *qr, QDP_String *md, QDP_F_Real *field);
int QDP_F_write_R( QDP_Writer *qw, QDP_String *md, QDP_F_Real *field);
int QDP_D_read_R( QDP_Reader *qr, QDP_String *md, QDP_D_Real *field);
int QDP_D_write_R( QDP_Writer *qw, QDP_String *md, QDP_D_Real *field);
int QDP_F_read_C( QDP_Reader *qr, QDP_String *md, QDP_F_Complex *field);
int QDP_F_write_C( QDP_Writer *qw, QDP_String *md, QDP_F_Complex *field);
int QDP_D_read_C( QDP_Reader *qr, QDP_String *md, QDP_D_Complex *field);
int QDP_D_write_C( QDP_Writer *qw, QDP_String *md, QDP_D_Complex *field);
int QDP_F2_read_V( QDP_Reader *qr, QDP_String *md, QDP_F2_ColorVector *field);
int QDP_F2_write_V( QDP_Writer *qw, QDP_String *md, QDP_F2_ColorVector *field);
int QDP_F3_read_V( QDP_Reader *qr, QDP_String *md, QDP_F3_ColorVector *field);
int QDP_F3_write_V( QDP_Writer *qw, QDP_String *md, QDP_F3_ColorVector *field);
int QDP_FN_read_V(int nc,  QDP_Reader *qr, QDP_String *md, QDP_FN_ColorVector *field);
int QDP_FN_write_V(int nc,  QDP_Writer *qw, QDP_String *md, QDP_FN_ColorVector *field);
int QDP_D2_read_V( QDP_Reader *qr, QDP_String *md, QDP_D2_ColorVector *field);
int QDP_D2_write_V( QDP_Writer *qw, QDP_String *md, QDP_D2_ColorVector *field);
int QDP_D3_read_V( QDP_Reader *qr, QDP_String *md, QDP_D3_ColorVector *field);
int QDP_D3_write_V( QDP_Writer *qw, QDP_String *md, QDP_D3_ColorVector *field);
int QDP_DN_read_V(int nc,  QDP_Reader *qr, QDP_String *md, QDP_DN_ColorVector *field);
int QDP_DN_write_V(int nc,  QDP_Writer *qw, QDP_String *md, QDP_DN_ColorVector *field);
int QDP_F2_read_H( QDP_Reader *qr, QDP_String *md, QDP_F2_HalfFermion *field);
int QDP_F2_write_H( QDP_Writer *qw, QDP_String *md, QDP_F2_HalfFermion *field);
int QDP_F3_read_H( QDP_Reader *qr, QDP_String *md, QDP_F3_HalfFermion *field);
int QDP_F3_write_H( QDP_Writer *qw, QDP_String *md, QDP_F3_HalfFermion *field);
int QDP_FN_read_H(int nc,  QDP_Reader *qr, QDP_String *md, QDP_FN_HalfFermion *field);
int QDP_FN_write_H(int nc,  QDP_Writer *qw, QDP_String *md, QDP_FN_HalfFermion *field);
int QDP_D2_read_H( QDP_Reader *qr, QDP_String *md, QDP_D2_HalfFermion *field);
int QDP_D2_write_H( QDP_Writer *qw, QDP_String *md, QDP_D2_HalfFermion *field);
int QDP_D3_read_H( QDP_Reader *qr, QDP_String *md, QDP_D3_HalfFermion *field);
int QDP_D3_write_H( QDP_Writer *qw, QDP_String *md, QDP_D3_HalfFermion *field);
int QDP_DN_read_H(int nc,  QDP_Reader *qr, QDP_String *md, QDP_DN_HalfFermion *field);
int QDP_DN_write_H(int nc,  QDP_Writer *qw, QDP_String *md, QDP_DN_HalfFermion *field);
int QDP_F2_read_D( QDP_Reader *qr, QDP_String *md, QDP_F2_DiracFermion *field);
int QDP_F2_write_D( QDP_Writer *qw, QDP_String *md, QDP_F2_DiracFermion *field);
int QDP_F3_read_D( QDP_Reader *qr, QDP_String *md, QDP_F3_DiracFermion *field);
int QDP_F3_write_D( QDP_Writer *qw, QDP_String *md, QDP_F3_DiracFermion *field);
int QDP_FN_read_D(int nc,  QDP_Reader *qr, QDP_String *md, QDP_FN_DiracFermion *field);
int QDP_FN_write_D(int nc,  QDP_Writer *qw, QDP_String *md, QDP_FN_DiracFermion *field);
int QDP_D2_read_D( QDP_Reader *qr, QDP_String *md, QDP_D2_DiracFermion *field);
int QDP_D2_write_D( QDP_Writer *qw, QDP_String *md, QDP_D2_DiracFermion *field);
int QDP_D3_read_D( QDP_Reader *qr, QDP_String *md, QDP_D3_DiracFermion *field);
int QDP_D3_write_D( QDP_Writer *qw, QDP_String *md, QDP_D3_DiracFermion *field);
int QDP_DN_read_D(int nc,  QDP_Reader *qr, QDP_String *md, QDP_DN_DiracFermion *field);
int QDP_DN_write_D(int nc,  QDP_Writer *qw, QDP_String *md, QDP_DN_DiracFermion *field);
int QDP_F2_read_M( QDP_Reader *qr, QDP_String *md, QDP_F2_ColorMatrix *field);
int QDP_F2_write_M( QDP_Writer *qw, QDP_String *md, QDP_F2_ColorMatrix *field);
int QDP_F3_read_M( QDP_Reader *qr, QDP_String *md, QDP_F3_ColorMatrix *field);
int QDP_F3_write_M( QDP_Writer *qw, QDP_String *md, QDP_F3_ColorMatrix *field);
int QDP_FN_read_M(int nc,  QDP_Reader *qr, QDP_String *md, QDP_FN_ColorMatrix *field);
int QDP_FN_write_M(int nc,  QDP_Writer *qw, QDP_String *md, QDP_FN_ColorMatrix *field);
int QDP_D2_read_M( QDP_Reader *qr, QDP_String *md, QDP_D2_ColorMatrix *field);
int QDP_D2_write_M( QDP_Writer *qw, QDP_String *md, QDP_D2_ColorMatrix *field);
int QDP_D3_read_M( QDP_Reader *qr, QDP_String *md, QDP_D3_ColorMatrix *field);
int QDP_D3_write_M( QDP_Writer *qw, QDP_String *md, QDP_D3_ColorMatrix *field);
int QDP_DN_read_M(int nc,  QDP_Reader *qr, QDP_String *md, QDP_DN_ColorMatrix *field);
int QDP_DN_write_M(int nc,  QDP_Writer *qw, QDP_String *md, QDP_DN_ColorMatrix *field);
int QDP_F2_read_P( QDP_Reader *qr, QDP_String *md, QDP_F2_DiracPropagator *field);
int QDP_F2_write_P( QDP_Writer *qw, QDP_String *md, QDP_F2_DiracPropagator *field);
int QDP_F3_read_P( QDP_Reader *qr, QDP_String *md, QDP_F3_DiracPropagator *field);
int QDP_F3_write_P( QDP_Writer *qw, QDP_String *md, QDP_F3_DiracPropagator *field);
int QDP_FN_read_P(int nc,  QDP_Reader *qr, QDP_String *md, QDP_FN_DiracPropagator *field);
int QDP_FN_write_P(int nc,  QDP_Writer *qw, QDP_String *md, QDP_FN_DiracPropagator *field);
int QDP_D2_read_P( QDP_Reader *qr, QDP_String *md, QDP_D2_DiracPropagator *field);
int QDP_D2_write_P( QDP_Writer *qw, QDP_String *md, QDP_D2_DiracPropagator *field);
int QDP_D3_read_P( QDP_Reader *qr, QDP_String *md, QDP_D3_DiracPropagator *field);
int QDP_D3_write_P( QDP_Writer *qw, QDP_String *md, QDP_D3_DiracPropagator *field);
int QDP_DN_read_P(int nc,  QDP_Reader *qr, QDP_String *md, QDP_DN_DiracPropagator *field);
int QDP_DN_write_P(int nc,  QDP_Writer *qw, QDP_String *md, QDP_DN_DiracPropagator *field);

/* Read and write arrays of fields */

int QDP_vread_S( QDP_Reader *qr, QDP_String *md,
		       QDP_RandomState *field[], int n);
int QDP_vwrite_S( QDP_Writer *qw, QDP_String *md,
			QDP_RandomState *field[], int n);
int QDP_vread_I( QDP_Reader *qr, QDP_String *md,
		       QDP_Int *field[], int n);
int QDP_vwrite_I( QDP_Writer *qw, QDP_String *md,
			QDP_Int *field[], int n);
int QDP_F_vread_R( QDP_Reader *qr, QDP_String *md,
		       QDP_F_Real *field[], int n);
int QDP_F_vwrite_R( QDP_Writer *qw, QDP_String *md,
			QDP_F_Real *field[], int n);
int QDP_D_vread_R( QDP_Reader *qr, QDP_String *md,
		       QDP_D_Real *field[], int n);
int QDP_D_vwrite_R( QDP_Writer *qw, QDP_String *md,
			QDP_D_Real *field[], int n);
int QDP_F_vread_C( QDP_Reader *qr, QDP_String *md,
		       QDP_F_Complex *field[], int n);
int QDP_F_vwrite_C( QDP_Writer *qw, QDP_String *md,
			QDP_F_Complex *field[], int n);
int QDP_D_vread_C( QDP_Reader *qr, QDP_String *md,
		       QDP_D_Complex *field[], int n);
int QDP_D_vwrite_C( QDP_Writer *qw, QDP_String *md,
			QDP_D_Complex *field[], int n);
int QDP_F2_vread_V( QDP_Reader *qr, QDP_String *md,
		       QDP_F2_ColorVector *field[], int n);
int QDP_F2_vwrite_V( QDP_Writer *qw, QDP_String *md,
			QDP_F2_ColorVector *field[], int n);
int QDP_F3_vread_V( QDP_Reader *qr, QDP_String *md,
		       QDP_F3_ColorVector *field[], int n);
int QDP_F3_vwrite_V( QDP_Writer *qw, QDP_String *md,
			QDP_F3_ColorVector *field[], int n);
int QDP_FN_vread_V(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_FN_ColorVector *field[], int n);
int QDP_FN_vwrite_V(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_FN_ColorVector *field[], int n);
int QDP_D2_vread_V( QDP_Reader *qr, QDP_String *md,
		       QDP_D2_ColorVector *field[], int n);
int QDP_D2_vwrite_V( QDP_Writer *qw, QDP_String *md,
			QDP_D2_ColorVector *field[], int n);
int QDP_D3_vread_V( QDP_Reader *qr, QDP_String *md,
		       QDP_D3_ColorVector *field[], int n);
int QDP_D3_vwrite_V( QDP_Writer *qw, QDP_String *md,
			QDP_D3_ColorVector *field[], int n);
int QDP_DN_vread_V(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_DN_ColorVector *field[], int n);
int QDP_DN_vwrite_V(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_DN_ColorVector *field[], int n);
int QDP_F2_vread_H( QDP_Reader *qr, QDP_String *md,
		       QDP_F2_HalfFermion *field[], int n);
int QDP_F2_vwrite_H( QDP_Writer *qw, QDP_String *md,
			QDP_F2_HalfFermion *field[], int n);
int QDP_F3_vread_H( QDP_Reader *qr, QDP_String *md,
		       QDP_F3_HalfFermion *field[], int n);
int QDP_F3_vwrite_H( QDP_Writer *qw, QDP_String *md,
			QDP_F3_HalfFermion *field[], int n);
int QDP_FN_vread_H(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_FN_HalfFermion *field[], int n);
int QDP_FN_vwrite_H(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_FN_HalfFermion *field[], int n);
int QDP_D2_vread_H( QDP_Reader *qr, QDP_String *md,
		       QDP_D2_HalfFermion *field[], int n);
int QDP_D2_vwrite_H( QDP_Writer *qw, QDP_String *md,
			QDP_D2_HalfFermion *field[], int n);
int QDP_D3_vread_H( QDP_Reader *qr, QDP_String *md,
		       QDP_D3_HalfFermion *field[], int n);
int QDP_D3_vwrite_H( QDP_Writer *qw, QDP_String *md,
			QDP_D3_HalfFermion *field[], int n);
int QDP_DN_vread_H(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_DN_HalfFermion *field[], int n);
int QDP_DN_vwrite_H(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_DN_HalfFermion *field[], int n);
int QDP_F2_vread_D( QDP_Reader *qr, QDP_String *md,
		       QDP_F2_DiracFermion *field[], int n);
int QDP_F2_vwrite_D( QDP_Writer *qw, QDP_String *md,
			QDP_F2_DiracFermion *field[], int n);
int QDP_F3_vread_D( QDP_Reader *qr, QDP_String *md,
		       QDP_F3_DiracFermion *field[], int n);
int QDP_F3_vwrite_D( QDP_Writer *qw, QDP_String *md,
			QDP_F3_DiracFermion *field[], int n);
int QDP_FN_vread_D(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_FN_DiracFermion *field[], int n);
int QDP_FN_vwrite_D(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_FN_DiracFermion *field[], int n);
int QDP_D2_vread_D( QDP_Reader *qr, QDP_String *md,
		       QDP_D2_DiracFermion *field[], int n);
int QDP_D2_vwrite_D( QDP_Writer *qw, QDP_String *md,
			QDP_D2_DiracFermion *field[], int n);
int QDP_D3_vread_D( QDP_Reader *qr, QDP_String *md,
		       QDP_D3_DiracFermion *field[], int n);
int QDP_D3_vwrite_D( QDP_Writer *qw, QDP_String *md,
			QDP_D3_DiracFermion *field[], int n);
int QDP_DN_vread_D(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_DN_DiracFermion *field[], int n);
int QDP_DN_vwrite_D(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_DN_DiracFermion *field[], int n);
int QDP_F2_vread_M( QDP_Reader *qr, QDP_String *md,
		       QDP_F2_ColorMatrix *field[], int n);
int QDP_F2_vwrite_M( QDP_Writer *qw, QDP_String *md,
			QDP_F2_ColorMatrix *field[], int n);
int QDP_F3_vread_M( QDP_Reader *qr, QDP_String *md,
		       QDP_F3_ColorMatrix *field[], int n);
int QDP_F3_vwrite_M( QDP_Writer *qw, QDP_String *md,
			QDP_F3_ColorMatrix *field[], int n);
int QDP_FN_vread_M(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_FN_ColorMatrix *field[], int n);
int QDP_FN_vwrite_M(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_FN_ColorMatrix *field[], int n);
int QDP_D2_vread_M( QDP_Reader *qr, QDP_String *md,
		       QDP_D2_ColorMatrix *field[], int n);
int QDP_D2_vwrite_M( QDP_Writer *qw, QDP_String *md,
			QDP_D2_ColorMatrix *field[], int n);
int QDP_D3_vread_M( QDP_Reader *qr, QDP_String *md,
		       QDP_D3_ColorMatrix *field[], int n);
int QDP_D3_vwrite_M( QDP_Writer *qw, QDP_String *md,
			QDP_D3_ColorMatrix *field[], int n);
int QDP_DN_vread_M(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_DN_ColorMatrix *field[], int n);
int QDP_DN_vwrite_M(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_DN_ColorMatrix *field[], int n);
int QDP_F2_vread_P( QDP_Reader *qr, QDP_String *md,
		       QDP_F2_DiracPropagator *field[], int n);
int QDP_F2_vwrite_P( QDP_Writer *qw, QDP_String *md,
			QDP_F2_DiracPropagator *field[], int n);
int QDP_F3_vread_P( QDP_Reader *qr, QDP_String *md,
		       QDP_F3_DiracPropagator *field[], int n);
int QDP_F3_vwrite_P( QDP_Writer *qw, QDP_String *md,
			QDP_F3_DiracPropagator *field[], int n);
int QDP_FN_vread_P(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_FN_DiracPropagator *field[], int n);
int QDP_FN_vwrite_P(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_FN_DiracPropagator *field[], int n);
int QDP_D2_vread_P( QDP_Reader *qr, QDP_String *md,
		       QDP_D2_DiracPropagator *field[], int n);
int QDP_D2_vwrite_P( QDP_Writer *qw, QDP_String *md,
			QDP_D2_DiracPropagator *field[], int n);
int QDP_D3_vread_P( QDP_Reader *qr, QDP_String *md,
		       QDP_D3_DiracPropagator *field[], int n);
int QDP_D3_vwrite_P( QDP_Writer *qw, QDP_String *md,
			QDP_D3_DiracPropagator *field[], int n);
int QDP_DN_vread_P(int nc,  QDP_Reader *qr, QDP_String *md,
		       QDP_DN_DiracPropagator *field[], int n);
int QDP_DN_vwrite_P(int nc,  QDP_Writer *qw, QDP_String *md,
			QDP_DN_DiracPropagator *field[], int n);

/* Read and write global array */

int QDP_vread_s( QDP_Reader *qr, QDP_String *md,
			  QLA_RandomState *field, int n);
int QDP_vwrite_s( QDP_Writer *qw, QDP_String *md,
			   QLA_RandomState *field, int n);
int QDP_vread_i( QDP_Reader *qr, QDP_String *md,
			  QLA_Int *field, int n);
int QDP_vwrite_i( QDP_Writer *qw, QDP_String *md,
			   QLA_Int *field, int n);
int QDP_F_vread_r( QDP_Reader *qr, QDP_String *md,
			  QLA_F_Real *field, int n);
int QDP_F_vwrite_r( QDP_Writer *qw, QDP_String *md,
			   QLA_F_Real *field, int n);
int QDP_D_vread_r( QDP_Reader *qr, QDP_String *md,
			  QLA_D_Real *field, int n);
int QDP_D_vwrite_r( QDP_Writer *qw, QDP_String *md,
			   QLA_D_Real *field, int n);
int QDP_F_vread_c( QDP_Reader *qr, QDP_String *md,
			  QLA_F_Complex *field, int n);
int QDP_F_vwrite_c( QDP_Writer *qw, QDP_String *md,
			   QLA_F_Complex *field, int n);
int QDP_D_vread_c( QDP_Reader *qr, QDP_String *md,
			  QLA_D_Complex *field, int n);
int QDP_D_vwrite_c( QDP_Writer *qw, QDP_String *md,
			   QLA_D_Complex *field, int n);
int QDP_F2_vread_v( QDP_Reader *qr, QDP_String *md,
			  QLA_F2_ColorVector *field, int n);
int QDP_F2_vwrite_v( QDP_Writer *qw, QDP_String *md,
			   QLA_F2_ColorVector *field, int n);
int QDP_F3_vread_v( QDP_Reader *qr, QDP_String *md,
			  QLA_F3_ColorVector *field, int n);
int QDP_F3_vwrite_v( QDP_Writer *qw, QDP_String *md,
			   QLA_F3_ColorVector *field, int n);
int QDP_FN_vread_v(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_FN_ColorVector *field, int n);
int QDP_FN_vwrite_v(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_FN_ColorVector *field, int n);
int QDP_D2_vread_v( QDP_Reader *qr, QDP_String *md,
			  QLA_D2_ColorVector *field, int n);
int QDP_D2_vwrite_v( QDP_Writer *qw, QDP_String *md,
			   QLA_D2_ColorVector *field, int n);
int QDP_D3_vread_v( QDP_Reader *qr, QDP_String *md,
			  QLA_D3_ColorVector *field, int n);
int QDP_D3_vwrite_v( QDP_Writer *qw, QDP_String *md,
			   QLA_D3_ColorVector *field, int n);
int QDP_DN_vread_v(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_DN_ColorVector *field, int n);
int QDP_DN_vwrite_v(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_DN_ColorVector *field, int n);
int QDP_F2_vread_h( QDP_Reader *qr, QDP_String *md,
			  QLA_F2_HalfFermion *field, int n);
int QDP_F2_vwrite_h( QDP_Writer *qw, QDP_String *md,
			   QLA_F2_HalfFermion *field, int n);
int QDP_F3_vread_h( QDP_Reader *qr, QDP_String *md,
			  QLA_F3_HalfFermion *field, int n);
int QDP_F3_vwrite_h( QDP_Writer *qw, QDP_String *md,
			   QLA_F3_HalfFermion *field, int n);
int QDP_FN_vread_h(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_FN_HalfFermion *field, int n);
int QDP_FN_vwrite_h(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_FN_HalfFermion *field, int n);
int QDP_D2_vread_h( QDP_Reader *qr, QDP_String *md,
			  QLA_D2_HalfFermion *field, int n);
int QDP_D2_vwrite_h( QDP_Writer *qw, QDP_String *md,
			   QLA_D2_HalfFermion *field, int n);
int QDP_D3_vread_h( QDP_Reader *qr, QDP_String *md,
			  QLA_D3_HalfFermion *field, int n);
int QDP_D3_vwrite_h( QDP_Writer *qw, QDP_String *md,
			   QLA_D3_HalfFermion *field, int n);
int QDP_DN_vread_h(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_DN_HalfFermion *field, int n);
int QDP_DN_vwrite_h(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_DN_HalfFermion *field, int n);
int QDP_F2_vread_d( QDP_Reader *qr, QDP_String *md,
			  QLA_F2_DiracFermion *field, int n);
int QDP_F2_vwrite_d( QDP_Writer *qw, QDP_String *md,
			   QLA_F2_DiracFermion *field, int n);
int QDP_F3_vread_d( QDP_Reader *qr, QDP_String *md,
			  QLA_F3_DiracFermion *field, int n);
int QDP_F3_vwrite_d( QDP_Writer *qw, QDP_String *md,
			   QLA_F3_DiracFermion *field, int n);
int QDP_FN_vread_d(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_FN_DiracFermion *field, int n);
int QDP_FN_vwrite_d(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_FN_DiracFermion *field, int n);
int QDP_D2_vread_d( QDP_Reader *qr, QDP_String *md,
			  QLA_D2_DiracFermion *field, int n);
int QDP_D2_vwrite_d( QDP_Writer *qw, QDP_String *md,
			   QLA_D2_DiracFermion *field, int n);
int QDP_D3_vread_d( QDP_Reader *qr, QDP_String *md,
			  QLA_D3_DiracFermion *field, int n);
int QDP_D3_vwrite_d( QDP_Writer *qw, QDP_String *md,
			   QLA_D3_DiracFermion *field, int n);
int QDP_DN_vread_d(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_DN_DiracFermion *field, int n);
int QDP_DN_vwrite_d(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_DN_DiracFermion *field, int n);
int QDP_F2_vread_m( QDP_Reader *qr, QDP_String *md,
			  QLA_F2_ColorMatrix *field, int n);
int QDP_F2_vwrite_m( QDP_Writer *qw, QDP_String *md,
			   QLA_F2_ColorMatrix *field, int n);
int QDP_F3_vread_m( QDP_Reader *qr, QDP_String *md,
			  QLA_F3_ColorMatrix *field, int n);
int QDP_F3_vwrite_m( QDP_Writer *qw, QDP_String *md,
			   QLA_F3_ColorMatrix *field, int n);
int QDP_FN_vread_m(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_FN_ColorMatrix *field, int n);
int QDP_FN_vwrite_m(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_FN_ColorMatrix *field, int n);
int QDP_D2_vread_m( QDP_Reader *qr, QDP_String *md,
			  QLA_D2_ColorMatrix *field, int n);
int QDP_D2_vwrite_m( QDP_Writer *qw, QDP_String *md,
			   QLA_D2_ColorMatrix *field, int n);
int QDP_D3_vread_m( QDP_Reader *qr, QDP_String *md,
			  QLA_D3_ColorMatrix *field, int n);
int QDP_D3_vwrite_m( QDP_Writer *qw, QDP_String *md,
			   QLA_D3_ColorMatrix *field, int n);
int QDP_DN_vread_m(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_DN_ColorMatrix *field, int n);
int QDP_DN_vwrite_m(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_DN_ColorMatrix *field, int n);
int QDP_F2_vread_p( QDP_Reader *qr, QDP_String *md,
			  QLA_F2_DiracPropagator *field, int n);
int QDP_F2_vwrite_p( QDP_Writer *qw, QDP_String *md,
			   QLA_F2_DiracPropagator *field, int n);
int QDP_F3_vread_p( QDP_Reader *qr, QDP_String *md,
			  QLA_F3_DiracPropagator *field, int n);
int QDP_F3_vwrite_p( QDP_Writer *qw, QDP_String *md,
			   QLA_F3_DiracPropagator *field, int n);
int QDP_FN_vread_p(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_FN_DiracPropagator *field, int n);
int QDP_FN_vwrite_p(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_FN_DiracPropagator *field, int n);
int QDP_D2_vread_p( QDP_Reader *qr, QDP_String *md,
			  QLA_D2_DiracPropagator *field, int n);
int QDP_D2_vwrite_p( QDP_Writer *qw, QDP_String *md,
			   QLA_D2_DiracPropagator *field, int n);
int QDP_D3_vread_p( QDP_Reader *qr, QDP_String *md,
			  QLA_D3_DiracPropagator *field, int n);
int QDP_D3_vwrite_p( QDP_Writer *qw, QDP_String *md,
			   QLA_D3_DiracPropagator *field, int n);
int QDP_DN_vread_p(int nc,  QDP_Reader *qr, QDP_String *md,
			  QLA_DN_DiracPropagator *field, int n);
int QDP_DN_vwrite_p(int nc,  QDP_Writer *qw, QDP_String *md,
			   QLA_DN_DiracPropagator *field, int n);

#ifdef __cplusplus
}
#endif

#endif /* _QDPIO_H */
